﻿/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

using System;
using System.Collections.ObjectModel;
using Xamarin.Forms;

namespace StopWatch
{
    /// <summary>
    /// StopWatch app's main page, logic part
    /// </summary>
    public partial class MainPage : ContentPage
    {
        private static System.Diagnostics.Stopwatch stopWatch;
        private StopwatchState swState = StopwatchState.init;
        private int lapIdx = 1;
        private string timeCache = "00:00:00";
        private char[] splitChars = { '.' };
        /// <summary>
        /// The constructor of MainPage
        /// </summary>
        /// <param name="screenWidth">The Screen's Width </param>
        /// <param name="screenHeight">The Screen's Height</param>
        public MainPage(int screenWidth, int screenHeight)
        {
            InitializeComponent(screenWidth, screenHeight);
            // Register start button's click event
            var startButtonTapGestureRecognizer = new TapGestureRecognizer();
            startButtonTapGestureRecognizer.Tapped += (s, e) =>
            {
                StopwatchState curState = swState;

                if (curState == StopwatchState.init || curState == StopwatchState.stopped)
                {
                    startButton.Source = "button2.png";
                    labButton.Source = "button3.png";
                    swState = StopwatchState.started;
                    stopWatch.Start();
                    // TimerFunc will be invoked every 10 milliseconds, except it return false.
                    Device.StartTimer(new TimeSpan(0, 0, 0, 0, 10), TimerFunc);
                }
                else if (curState == StopwatchState.started)
                {
                    startButton.Source = "button1.png";
                    labButton.Source = "button4.png";
                    stopWatch.Stop();
                    swState = StopwatchState.stopped;
                }
            };
            startButton.GestureRecognizers.Add(startButtonTapGestureRecognizer);
            // Register lap button's click event
            var lapButtonTapGestureRecognizer = new TapGestureRecognizer();
            lapButtonTapGestureRecognizer.Tapped += (s, e) =>
            {
                StopwatchState curState = swState;
                if (curState == StopwatchState.started)
                {
                    // Get stopwatch's time string
                    string[] ret = stopWatch.Elapsed.ToString().Split(splitChars);
                    lapList.Add(new LapListItem(lapIdx, ret[0], ret[1].Substring(0, 2)));
                    lapIdx++;
                }
                else if (curState == StopwatchState.stopped)
                {
                    // If curState is stopped, then reset timer and widget's text.
                    stopWatch.Reset();
                    lapList.Clear();
                    labButton.Source = "button3.png";
                    timeLabel.Text = "00:00:00";
                    timeCache = "00:00:00";
                    minorLabel.Text = ".00";
                    lapIdx = 1;
                    swState = StopwatchState.init;
                }

                try
                {
                    int len = ((ObservableCollection<LapListItem>)(lapListView.ItemsSource)).Count;
                    lapListView.ScrollTo(((ObservableCollection<LapListItem>)(lapListView.ItemsSource))[len - 1], ScrollToPosition.End, false);
                }
                catch (Exception)
                {
                }
            };
            labButton.GestureRecognizers.Add(lapButtonTapGestureRecognizer);

            stopWatch = new System.Diagnostics.Stopwatch();
        }

        /// <summary>
        /// Timer callback function
        /// </summary>
        /// <returns>false: stop timer; true: continue the timer</returns>
        bool TimerFunc()
        {
            StopwatchState curState = swState;

            if (curState == StopwatchState.started)
            {
                string[] ret = stopWatch.Elapsed.ToString().Split(splitChars);
                if (0 != string.Compare(timeCache, ret[0]))
                {
                    timeCache = ret[0];
                    timeLabel.Text = ret[0];
                }

                minorLabel.Text = "." + ret[1].Substring(0, 2);
            }
            else
            {
                return false;
            }

            return true;
        }
    }

    /// <summary>
    /// StopWatch status
    /// </summary>
    internal enum StopwatchState
    {
        init,
        started,
        stopped
    }
}